package cn.iocoder.yudao.module.member.controller.admin.job;

import cn.hutool.json.JSONArray;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.quartz.core.handler.JobHandler;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScoreDetail.MemberUserScoreDetailDO;
import cn.iocoder.yudao.module.member.enums.MemberScoreStatueEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.service.memberUserScoreDetail.MemberUserScoreDetailService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 会员积分过期定时任务
 */
@Component
@Slf4j
public class MemberUserScoreExpireTask implements JobHandler {
    /**
     * 会员积分的过期时间为每日的0点整
     * 任务每天0点整运行，扫描当天0点过期的积分进行状态修改
     * @param param 参数
     * @return
     * @throws Exception
     */
    @Resource
    private MemberUserScoreDetailService scoreDetailService;

    @Resource
    private MemberUserScoreApi memberUserScoreApi;

    @Override
    public String execute(String param) throws Exception {
        log.info("member user score expire task running");
        LambdaQueryWrapper<MemberUserScoreDetailDO> wrapper = Wrappers.lambdaQuery();
        wrapper.in(MemberUserScoreDetailDO::getStatus, Lists.newArrayList(MemberScoreStatueEnum.AVAILABLE.getValue(),
                MemberScoreStatueEnum.PART_AVAILABLE.getValue()));
        wrapper.le(MemberUserScoreDetailDO::getExpireTime, DateUtils.getNextNDayStart(new Date(), 0));
        wrapper.orderByAsc(MemberUserScoreDetailDO::getCreateTime);
        List<MemberUserScoreDetailDO> todoList = scoreDetailService.list(wrapper);
        log.info("member user score expire task, to expire record count :{}", todoList.size());
        if (CollectionUtils.isEmpty(todoList)) {
            return "success";
        }
        for (MemberUserScoreDetailDO memberUserScoreDetailDO : todoList) {
            try {
                log.info("score expire, score detail id :{}", memberUserScoreDetailDO.getId());
                JSONArray logIds = memberUserScoreDetailDO.getExtParamByKey(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS);
                Map<String, Object> extParam = new HashMap<>();
                extParam.put("scoreLogIds", logIds.toList(Long.class));
                memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                                .memberId(memberUserScoreDetailDO.getMemberId())
                                .scoreCount(memberUserScoreDetailDO.getRemainCount())
                                .sourceType(ScoreSourceTypeEnum.SYSTEM_EXPIRED)
                                .extParam(extParam)
                        .build());
            } catch (Exception e) {
                log.error("member user score expire exception, data:{}", memberUserScoreDetailDO, e);
            }
        }
        log.info("member user score expire task finished");
        return "success";
    }
}
