package cn.iocoder.yudao.module.member.controller.app.auth.vo;

import cn.iocoder.yudao.framework.common.validation.InEnum;
import cn.iocoder.yudao.framework.common.validation.Mobile;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

/**
 * @author Administrator
 */
@ApiModel("用户 APP - 校验验证码 Request VO")
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class AppAuthCheckCodeReqVO {

    @ApiModelProperty(value = "手机号", example = "15601691234")
    @NotBlank(message = "{app.auth.mobile.not.blank}")
    // @Mobile(message = "{app.auth.mobile.format.err}")
    private String mobile;

    @ApiModelProperty(value = "手机验证码", required = true, example = "1024")
    @NotBlank(message = "{app.auth.captcha.not.blank}")
    @Length(min = 4, max = 6, message = "{app.auth.captcha.len.err}")
    @Pattern(regexp = "^[0-9]+$", message = "{app.auth.captcha.pattern}")
    private String code;

    @ApiModelProperty(value = "发送场景", example = "1", notes = "对应 SmsSceneEnum 枚举")
    @NotNull(message = "{app.sms.scene.not.blank}")
    @InEnum(value = SmsSceneEnum.class, message = "{app.sms.scene.not.range}")
    private Integer scene;

}
