package cn.iocoder.yudao.module.member.dal.dataobject.user;

import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.tenant.core.db.TenantBaseDO;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.*;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import java.util.Date;

/**
 * 会员用户 DO
 *
 * uk_mobile 索引：基于 {@link #mobile} 字段
 *
 * @author 捷道源码
 */
@TableName(value = "member_user", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class MemberUserDO extends TenantBaseDO {

    /**
     * 用户ID
     */
    @TableId
    private Long id;
    /**
     * 用户昵称
     */
    private String nickname;
    /**
     * 用户头像
     */
    private String avatar;
    /**
     * 帐号状态
     *
     * 枚举 {@link CommonStatusEnum}
     */
    private Integer status;

    /**
     * 手机
     */
    private String mobile;
    /**
     * 加密后的密码
     *
     * 因为目前使用 {@link BCryptPasswordEncoder} 加密器，所以无需自己处理 salt 盐
     */
    private String password;
    /**
     * 注册 IP
     */
    private String registerIp;
    /**
     * 最后登录IP
     */
    private String loginIp;
    /**
     * 最后登录时间
     */
    private Date loginDate;

    /**
     * 保函
     */
    private String backLetterImg;

    /**
     * 区号
     */
    private String areaCode;

    /**
     * 英文名称
     */

    private String  englishName;

    private String email;


    /**
     * 加密后的控货密码
     *
     * 因为目前使用 {@link BCryptPasswordEncoder} 加密器，所以无需自己处理 salt 盐
     */
    private String controlPassword;

    ///**
    // * 是否身份认证
    // */
    //@TableField(exist = false)
    //private Boolean isIdentityAuthtificated;
    ///**
    // * 是否企业认证
    // */
    //@TableField(exist = false)
    //private Boolean isEnterpriseAuthtificated;

    /**
     * 身份认证审核状态
     */
    @TableField(exist = false)
    private Integer identityAuditStatus;

    /**
     * 身份认证审核创建时间
     */
    @TableField(exist = false)
    private Date identityAuditCreateTime;

    /**
     * 身份认证审核审核时间
     */
    @TableField(exist = false)
    private Date identityAuditTime;

    /**
     * 身份认证审核失败原因
     */
    @TableField(exist = false)
    private String identityAuditRemark;

    /**
     * 真实姓名
     */
    @TableField(exist = false)
    private String identityName;


    /**
     * 企业认证审核状态
     */
    @TableField(exist = false)
    private Integer enterpriseAuditStatus;

    /**
     * 企业认证审核创建时间
     */
    @TableField(exist = false)
    private Date enterpriseAuditCreateTime;

    /**
     * 企业认证审核审核时间
     */
    @TableField(exist = false)
    private Date enterpriseAuditTime;

    /**
     * 企业认证审核失败原因
     */
    @TableField(exist = false)
    private String enterpriseAuditRemark;

    /**
     * 企业名称(中文)
     */
    @TableField(exist = false)
    private String enterpriseName;

    /**
     * 企业名称(英文)
     */
    @TableField(exist = false)
    private String enterpriseNameEn;


    /**
     * 公司简介
     */
    @TableField(exist = false)
    private String enterpriseDesc;

    /**
     * 公司官网
     */
    @TableField(exist = false)
    private String enterpriseWebsite;

    /**
     * 公司地址
     */
    @TableField(exist = false)
    private String enterpriseAddress;

    /**
     * 档口
     */
    @TableField(exist = false)
    private String enterpriseStall;
    /**
     * 品牌
     */
    @TableField(exist = false)
    private String enterpriseBrand;

    /**
     * 客户状态
     */
    private Integer customerStatus;

    /**
     * 生日
     */
    private Date birthday;
    /**
     * 部门
     */
    private String department;
    /**
     * 性别：0 女 1 男 2 保密
     */
    private Integer gender;
    /**
     * 工作职位
     */
    private String jobPosition;
    /**
     * 地址
     */
    private String address;


    /**
     * 后台客户成交状态
     */
    private Boolean isDeal;

    private String code;

    private Integer holdScore;
}
