package cn.iocoder.yudao.module.member.job;

import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.quartz.core.handler.JobHandler;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 会员积分过期定时任务
 */
@Component
@Slf4j
public class ScoreRuleExpireTask implements JobHandler {
    /**
     * 积分规则的过期时间为每日的0点整
     * 任务每天0点整运行，扫描当天0点过期的积分进行状态修改
     * @param param 参数
     * @return
     * @throws Exception
     */

    @Resource
    private ScoreRuleService scoreRuleService;

    @Override
    public String execute(String param) throws Exception {
        log.info("score rule expire task running");
        LambdaQuery<ScoreRuleDO> scoreRuleDOLambdaQuery = new LambdaQuery<>();
        scoreRuleDOLambdaQuery.eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue());
        scoreRuleDOLambdaQuery.le(ScoreRuleDO::getEndTime, DateUtils.getNextNDayStart(new Date(), 0));
        List<ScoreRuleDO> todoList = scoreRuleService.selectList(scoreRuleDOLambdaQuery);
        log.info("score rule expire task, to expire rule count :{}", todoList.size());
        if (CollectionUtils.isEmpty(todoList)) {
            return "success";
        } else {
            todoList.forEach(scoreRuleDO -> scoreRuleDO.setStatus(ScoreRuleStatusEnum.EXPIRED.getValue()));
            scoreRuleService.updateBatch(todoList);
        }
        return "success";
    }
}
