package cn.iocoder.yudao.module.member.listener;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.MemberScoreDetailExpireEvent;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScoreDetail.MemberUserScoreDetailDO;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.service.memberUserScoreDetail.MemberUserScoreDetailService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
@AllArgsConstructor
@Slf4j
public class MemberUserScoreDetailExpireListener {
    @Resource
    private MemberUserScoreDetailService scoreDetailService;
    @Resource
    private MemberUserScoreApi memberUserScoreApi;
    @Async
    @EventListener(MemberScoreDetailExpireEvent.class)
    public void listen(MemberScoreDetailExpireEvent event) {
        log.info("member user score expire listen,event:{}", JSONUtil.toJsonStr(event));
        if (CollectionUtils.isEmpty(event.getDetailIds())) {
            return;
        }
        //暂停一会确保那边事务执行完成
        ThreadUtil.safeSleep(3000L);
        LambdaQueryWrapper<MemberUserScoreDetailDO> wrapper = Wrappers.lambdaQuery();
        wrapper.in(MemberUserScoreDetailDO::getId, event.getDetailIds());
        wrapper.le(MemberUserScoreDetailDO::getExpireTime, new Date());
        List<MemberUserScoreDetailDO> todoList = scoreDetailService.list(wrapper);
        todoList.forEach(detail -> {
            try {
                log.info("score expire, score detail id :{}", detail.getId());
                JSONArray jsonArray = detail.getExtParamByKey(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS);
                Map<String, Object> extParam = new HashMap<>();
                extParam.put("scoreLogIds", jsonArray.toList(Long.class));
                memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                        .memberId(detail.getMemberId())
                        .scoreCount(detail.getRemainCount())
                        .sourceType(ScoreSourceTypeEnum.SYSTEM_EXPIRED)
                        .extParam(extParam)
                        .build());
            } catch (Exception e) {
                log.error("member user score expire exception, data:{}", detail, e);
            }
        });
        log.info("member user score expire listen finished");
    }
}
