package cn.iocoder.yudao.module.member.service.memberUserScoreDetail;

import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreDetailUpdateReqDto;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScoreDetail.MemberUserScoreDetailDO;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScoreDetailReleation.MemberUserScoreDetailReleationDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreDetail.MemberUserScoreDetailMapper;
import cn.iocoder.yudao.module.member.dto.ScoreDetailChangeDto;
import cn.iocoder.yudao.module.member.enums.MemberScoreStatueEnum;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import cn.iocoder.yudao.module.member.service.memberUserScoreDetailReleation.MemberUserScoreDetailReleationService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.addDays;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.getNextNDayStart;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.member.enums.MemberScoreStatueEnum.AVAILABLE;
import static cn.iocoder.yudao.module.member.enums.MemberScoreStatueEnum.PART_AVAILABLE;

/**
 * 会员积分详情 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreDetailServiceImpl extends AbstractService<MemberUserScoreDetailMapper, MemberUserScoreDetailDO> implements MemberUserScoreDetailService {
    @Autowired
    private MemberUserScoreDetailMapper detailMapper;
    @Autowired
    private MemberUserScoreDetailReleationService memberUserScoreDetailReleationService;
    @Autowired
    private SnowflakeGenerator snowflakeGenerator;

    @Override
    public List<ScoreDetailChangeDto> updateScoreDetail(MemberUserScoreDetailUpdateReqDto reqDto) {
        // 增加积分，只需要新增一条记录
        // 减少积分，使已有记录失效
        if (reqDto.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        if (reqDto.getSourceType().getReverseSource() != null && reqDto.getOperateType() == ScoreOperateTypeEnum.ADD) {
            //说明是回退流程新增加分 根据releationId 和source 获取之前操作过的detail
            //如果是回退流程减少积分 则走正常的扣减逻辑
            return dealReverseSourceDetail(reqDto);
        }
        if (reqDto.getOperateType() == ScoreOperateTypeEnum.ADD) {
            return update4Add(reqDto);
        }
        if (reqDto.getOperateType() == ScoreOperateTypeEnum.REDUCE) {
            return update4Reduce(reqDto);
        }
        return Collections.emptyList();
    }

    private List<ScoreDetailChangeDto> dealReverseSourceDetail(MemberUserScoreDetailUpdateReqDto reqDto) {
        if (StringUtils.isBlank(reqDto.getReleationId())) {
            throw exception(REVERSE_SOURCE_NO_RELEATION_ID);
        }
        List<MemberUserScoreDetailReleationDO> releationDOList = memberUserScoreDetailReleationService.listByReleationIdAndSource(reqDto.getReleationId(),
                reqDto.getSourceType().getReverseSource().getValue());
        if (reqDto.getOperateType() == ScoreOperateTypeEnum.ADD) {
            //积分恢复
            dealReverseSourceAdd(releationDOList, reqDto);
        }
        return releationDOList.stream().map(e ->ScoreDetailChangeDto.builder().detailId(e.getDetailId()).scoreCount(e.getScoreCount()).build())
                .collect(Collectors.toList());
    }

    private void dealReverseSourceAdd(List<MemberUserScoreDetailReleationDO> releationDOList, MemberUserScoreDetailUpdateReqDto reqDto) {
        List<Long> detailIds = releationDOList.stream().map(MemberUserScoreDetailReleationDO::getDetailId).collect(Collectors.toList());
        Map<Long, MemberUserScoreDetailReleationDO> detailIdReleationDoMap = releationDOList.stream().collect(Collectors.toMap(MemberUserScoreDetailReleationDO::getDetailId, Function.identity(), (c1, c2) -> c1));
        LambdaQueryWrapper<MemberUserScoreDetailDO> wrappers = Wrappers.lambdaQuery();
        wrappers.in(MemberUserScoreDetailDO::getId, detailIds);
        List<MemberUserScoreDetailDO> detailDOList = this.list(wrappers);
        List<MemberUserScoreDetailDO> saveDetailList =  new ArrayList<>();
        detailDOList.forEach(detail -> {
            MemberUserScoreDetailReleationDO releationDO = detailIdReleationDoMap.get(detail.getId());
            if (releationDO == null) {
                return;
            }
            MemberUserScoreDetailDO updateData = new MemberUserScoreDetailDO();
            updateData.setId(detail.getId());
            updateData.setRemainCount(detail.getRemainCount() + releationDO.getScoreCount());
            updateData.setStatus(Objects.equals(detail.getScoreCount(), updateData.getRemainCount()) ? AVAILABLE.getValue() : PART_AVAILABLE.getValue());
            updateData.setExtParam(addScoreLogId(detail.getExtParam(), reqDto.getScoreLogId()));
            saveDetailList.add(updateData);
        });
        this.saveOrUpdateBatch(saveDetailList);
    }

    private List<ScoreDetailChangeDto> update4Reduce(MemberUserScoreDetailUpdateReqDto reqDto) {
        Long scoreCount = detailMapper.getScoreTotalByStatus(reqDto.getMemberId(), Lists.newArrayList(AVAILABLE.getValue(), MemberScoreStatueEnum.PART_AVAILABLE.getValue()));
        if (scoreCount == null || scoreCount < reqDto.getScoreCount()) {
            throw exception(MEMBER_SCORE_NOT_ENOUGH);
        }
        LambdaQueryWrapper<MemberUserScoreDetailDO> wrappers = Wrappers.lambdaQuery();
        wrappers.eq(MemberUserScoreDetailDO::getMemberId, reqDto.getMemberId());
        wrappers.in(MemberUserScoreDetailDO::getStatus, Lists.newArrayList(AVAILABLE.getValue(), MemberScoreStatueEnum.PART_AVAILABLE.getValue()));
        List<MemberUserScoreDetailDO> detailList = this.list(wrappers);
        detailList = sort(detailList);
        int needCount = reqDto.getScoreCount();
        List<MemberUserScoreDetailDO> updateDetailList = new ArrayList<>();
        List<ScoreDetailChangeDto> scoreDetailChangeDtos = new ArrayList<>();
        for (MemberUserScoreDetailDO memberUserScoreDetailDO : detailList) {
            if (needCount <= 0) {
                break;
            }
            Integer usedScoreCount;
            if (memberUserScoreDetailDO.getRemainCount() <= needCount) {
                usedScoreCount = memberUserScoreDetailDO.getRemainCount();
                needCount -= usedScoreCount;
                memberUserScoreDetailDO.setRemainCount(0);
                memberUserScoreDetailDO.setStatus(MemberScoreStatueEnum.NOT_AVAILABLE.getValue());
                memberUserScoreDetailDO.setExtParam(addScoreLogId(memberUserScoreDetailDO.getExtParam(), reqDto.getScoreLogId()));
                updateDetailList.add(memberUserScoreDetailDO);
            } else {
                usedScoreCount = needCount;
                memberUserScoreDetailDO.setRemainCount(memberUserScoreDetailDO.getRemainCount() - usedScoreCount);
                needCount = 0;
                memberUserScoreDetailDO.setStatus(MemberScoreStatueEnum.PART_AVAILABLE.getValue());
                memberUserScoreDetailDO.setExtParam(addScoreLogId(memberUserScoreDetailDO.getExtParam(), reqDto.getScoreLogId()));
                updateDetailList.add(memberUserScoreDetailDO);
            }
            scoreDetailChangeDtos.add(ScoreDetailChangeDto.builder().detailId(memberUserScoreDetailDO.getId()).scoreCount(usedScoreCount).build());
        }
        this.updateBatch(updateDetailList);
        return scoreDetailChangeDtos;
    }

    private List<MemberUserScoreDetailDO> sort(List<MemberUserScoreDetailDO> detailList) {
        // 使用优先级  expire time asc > 没有过期时间的积分
        Map<Boolean, List<MemberUserScoreDetailDO>> groupDetailMap = detailList.stream().collect(Collectors.groupingBy(e -> e.getExpireTime() != null));
        List<MemberUserScoreDetailDO> expireTimeDetail = groupDetailMap.getOrDefault(true, new ArrayList<>()).stream()
                .sorted(Comparator.comparing(MemberUserScoreDetailDO::getExpireTime)).collect(Collectors.toList());
        expireTimeDetail.addAll(groupDetailMap.getOrDefault(false, new ArrayList<>()));
        return expireTimeDetail;
    }

    private List<ScoreDetailChangeDto> update4Add(MemberUserScoreDetailUpdateReqDto reqDto) {
        MemberUserScoreDetailDO memberUserScoreDetailDO = new MemberUserScoreDetailDO();
        memberUserScoreDetailDO.setId(snowflakeGenerator.next());
        memberUserScoreDetailDO.setMemberId(reqDto.getMemberId());
        memberUserScoreDetailDO.setScoreCount(reqDto.getScoreCount());
        memberUserScoreDetailDO.setStatus(AVAILABLE.getValue());
        memberUserScoreDetailDO.setRemainCount(reqDto.getScoreCount());
        memberUserScoreDetailDO.setCreateTime(new Date());
        if (reqDto.getExpireDays() != null) {
            memberUserScoreDetailDO.setExpireTime(addDays(getNextNDayStart(memberUserScoreDetailDO.getCreateTime(), 1), reqDto.getExpireDays()));
        }
        Map<String, Object> extParma = new HashMap<>();
        extParma.put(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS.getKey(), Lists.newArrayList(reqDto.getScoreLogId()));
        memberUserScoreDetailDO.setExtParam(JSONUtil.toJsonStr(extParma));
        this.saveOrUpdate(memberUserScoreDetailDO);
        return Lists.newArrayList(ScoreDetailChangeDto.builder().detailId(memberUserScoreDetailDO.getId()).scoreCount(reqDto.getScoreCount()).build());
    }

    private String addScoreLogId(String extParam, Long scoreLogId) {
        JSONObject extParamJson = JSONUtil.parseObj(extParam);
        if (extParam == null ){
            extParamJson = new JSONObject();
        }
        List<Long> logIds = extParamJson.get(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS.getKey(), List.class);
        if (logIds == null) {
            logIds = new ArrayList<>();
        }
        logIds.add(scoreLogId);
        extParamJson.set(MemberUserScoreDetailDO.MemberUserScoreDetailExtKey.LOG_IDS.getKey(), logIds);
        return extParamJson.toStringPretty();
    }
}
