package cn.iocoder.yudao.module.member.service.memberUserScoreLog;

import cn.hutool.core.lang.generator.SnowflakeGenerator;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScoreLog.MemberUserScoreLogMapper;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogCreateReq;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogQueryVO;
import cn.iocoder.yudao.module.system.service.dict.DictTypeService;
import com.alibaba.excel.util.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.MEMBER_ID_IS_NULL;

/**
 * 会员积分日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreLogServiceImpl extends AbstractService<MemberUserScoreLogMapper, MemberUserScoreLogDO> implements MemberUserScoreLogService {

    @Resource
    private MemberUserScoreLogMapper userScoreLogMapper;

    @Resource
    private DictTypeService dictTypeService;

    @Resource
    private SnowflakeGenerator snowflakeGenerator;


    @Override
    public PageResult<MemberUserScoreLogBackVO> getPage(MemberUserScoreLogQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserScoreLogBackVO> list = userScoreLogMapper.getPageRecordList(start, size, query);
        int total = userScoreLogMapper.getPageCount(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public Long createScoreLog(MemberUserScoreLogCreateReq createReq) {
        if (createReq.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        MemberUserScoreLogDO userScoreLogDO = new MemberUserScoreLogDO();
        userScoreLogDO.setId(snowflakeGenerator.next());
        userScoreLogDO.setMemberId(createReq.getMemberId());
        userScoreLogDO.setScoreCount(createReq.getScoreCount());
        userScoreLogDO.setOperateType(createReq.getOperateType());
        userScoreLogDO.setSourceType(createReq.getSourceType());
        userScoreLogDO.setRuleId(createReq.getRuleId());
        if (createReq.getExtParam() != null) {
            userScoreLogDO.setExtParam(JSONUtil.toJsonStr(createReq.getExtParam()));
        }
        this.saveOrUpdate(userScoreLogDO);
        return userScoreLogDO.getId();
    }
}
