package cn.iocoder.yudao.module.member.service.userCardAuth;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.ecw.api.internalMessage.ClientInternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.member.dal.mysql.userOperationLog.UserOperationLogMapper;
import cn.iocoder.yudao.module.member.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.member.enums.UserAuditStatusEnum;
import cn.iocoder.yudao.module.member.enums.UserOperationLogTypeEnum;
import cn.iocoder.yudao.module.member.service.userOperationLog.UserOperationLogService;
import cn.iocoder.yudao.module.member.vo.userOperationLog.UserOperationLogCreateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.vo.userCardAuth.*;
import cn.iocoder.yudao.module.member.dal.dataobject.userCardAuth.UserCardAuthDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.member.convert.userCardAuth.UserCardAuthConvert;
import cn.iocoder.yudao.module.member.dal.mysql.userCardAuth.UserCardAuthMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.servlet.ServletUtils.getClientIP;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 用户身份证认证 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class UserCardAuthServiceImpl extends AbstractService<UserCardAuthMapper, UserCardAuthDO> implements UserCardAuthService {

    @Resource
    private UserCardAuthMapper userCardAuthMapper;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Resource
    private ClientInternalMessageApi clientInternalMessageApi;


    @Override
    public Long createUserCardAuth(UserCardAuthCreateReqVO createReqVO, boolean isAdminManagerUpload) {

        //审核中和审核通过已存在不能提交

        List<UserCardAuthCreateReqVO> userCardAuthCreateReqVOS = userCardAuthMapper.selectUserCardAuthList(createReqVO.getUserId());
        if (userCardAuthCreateReqVOS.size() > 0) {
            throw exception(ErrorCodeConstants.USER_CARD_AUTH_EXISTS);
        }

        UserCardAuthDO authDO =  userCardAuthMapper.selectByUserId(createReqVO.getUserId());
        //删除历史认证信息
        if (authDO != null) {
            userCardAuthMapper.deleteById(authDO.getId());
        }

        // 插入
        UserCardAuthDO userCardAuth = UserCardAuthConvert.INSTANCE.convert(createReqVO);
        if (isAdminManagerUpload) {
            userCardAuth.setStatus(UserAuditStatusEnum.AUDIT_SUCCESS.getValue());
            userCardAuth.setAuditTime(new Date());
        } else {
            userCardAuth.setStatus(UserAuditStatusEnum.AUDITING.getValue());
        }
        userCardAuthMapper.insert(userCardAuth);

        //站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(createReqVO.getUserId()))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.kyc.submit.title"))
                .content(I18nMessage.getMessage("app.message.kyc.submit.content"))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);

        //记录日志
        UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
        userOperationLogCreateReqVO
                .setUserId(Long.parseLong(userCardAuth.getCreator()))
                .setType(UserOperationLogTypeEnum.AUTHENTIFICATION.getValue())
                .setTitle(UserOperationLogTypeEnum.AUTHENTIFICATION.getDesc())
                .setIp(getClientIP())
                .setContent("实名认证已提交");

        userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);

        // 返回
        return userCardAuth.getId();
    }

    @Override
    public void updateUserCardAuth(UserCardAuthUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateUserCardAuthExists(updateReqVO.getId());
        // 更新
        UserCardAuthDO updateObj = UserCardAuthConvert.INSTANCE.convert(updateReqVO);
        userCardAuthMapper.updateById(updateObj);
    }

    @Override
    public void deleteUserCardAuth(Long id) {
        // 校验存在
        this.validateUserCardAuthExists(id);
        // 删除
        userCardAuthMapper.deleteById(id);
    }

    private void validateUserCardAuthExists(Long id) {
        if (userCardAuthMapper.selectById(id) == null) {
            throw exception(USER_CARD_AUTH_NOT_EXISTS);
        }
    }

    @Override
    public UserCardAuthDO getUserCardAuth(Long id) {
        return userCardAuthMapper.selectById(id);
    }

    @Override
    public List<UserCardAuthDO> getUserCardAuthList(Collection<Long> ids) {
        return userCardAuthMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<UserCardAuthDO> getUserCardAuthPage(UserCardAuthQueryVO query, PageVO page) {
        return userCardAuthMapper.selectPage(page, query);
    }

    @Override
    public List<UserCardAuthDO> getUserCardAuthList(UserCardAuthQueryVO query) {
        return userCardAuthMapper.selectList(query);
    }
}
